#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h> 
#include <math.h>

#ifndef F_CPU
#define F_CPU 1600000UL
#endif

#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)
#define TX_FREQ 100

int soundSum=0;
int tempSum=0;

//**********************************************************

void adc_init()
{
    // AREF = AVCC
    ADMUX = (1<<REFS0);				//(0<<REFS1)| 
    // ADC Enable and prescaler of 128
    ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
}

// read adc value
uint16_t adc_read(uint8_t ch)
{
    // select the corresponding channel 0~7
    ch &= 0b00000111;  // AND operation with 7
    ADMUX = (ADMUX & 0xF8)|ch;    
    // start single conversion
    // write '1' to ADSC
    ADCSRA |= (1<<ADSC);
    // wait for conversion to complete
    // ADSC becomes '0' again
    while(ADCSRA & (1<<ADSC));
    return (ADC);
}


void usart_init()
{
	UCSR0B = (1 << TXEN0);   // Turn on the transmission and reception circuitry
	UCSR0C = (1 << UCSZ00) | (1 << UCSZ01) | (1<<UPM01); // Use 8-bit character sizes and Even Parity

	UBRR0H = (BAUD_PRESCALE >> 8); // Load upper 8-bits of the baud rate value into the high byte of the UBRR register
	UBRR0L = BAUD_PRESCALE; // Load lower 8-bits of the baud rate value into the low byte of the UBRR register
}

void USART_Transmit( unsigned char data )
{
UDR0=0;//clear UART buffer
/* Wait for empty transmit buffer */
while ( !( UCSR0A & (1<<UDRE0)) );
/* Put data into buffer, sends the data */
UDR0 = data;
}

void USART_Transmit_String(char *str) //take address vaue of the string in pionter *str
{
	 int i=0;
	 while(str[i]!='\0') // loop will go on till the NULL charaters is soon in string 
	 {
		 USART_Transmit(str[i]); // sending data on CD byte by byte
		 i++;
	 }
	 return;
}

int main ( void )
{
	DDRA = 0<<PA7;  // Declare temperature sensor as input
	DDRA = 0<<PA1;	//Declare opAmp output as input to ADC Channel 1
	DDRA = 0<<PA3;	//Declare DC Bias output as input to ADC Channel 3
	DDRA = 0<<PA5;  // Read Battery Volatge 
	DDRB = 1<<PB0;  // Declare Mosfet Gate Pin as Output
	
	int adcT_result;
	int adcS_result;
	int adcDC_result;
	int adcB_result;
	
	char str_temp [10];
	char str_sound [10];

	int temperature;
	int sound;
uint8_t elapsedTime = 0;
TCCR1B |= (1 << CS10) | (1 << CS12); // Set up timer at Fcpu/1024 = 15625
	
	adc_init(); 
	usart_init();
	
	while(1) // Loop forever
 	{
		adcB_result = adc_read(5);      // read adc value - Battery Volatge at PA5
		
		if (adcB_result >=660)		// Corresponds to 13V
		
			 PORTB &= ~(1<<PB0);
		
		else if (adcB_result <= 615)		// Corresponds to 12V
		
			PORTB |= (1<<PB0);

 		if (TCNT1 >= 1562)   //1 sec delay
        	{
       		TCNT1 = 0;
     			if (elapsedTime == TX_FREQ) //Calculates ADC for every 10 secs
     			{
			  itoa(soundSum/elapsedTime,str_sound,10); 
			  itoa(tempSum/elapsedTime,str_temp,10);

			  USART_Transmit('#'); 
			  USART_Transmit_String(str_temp);
			  USART_Transmit('|');
			  USART_Transmit_String(str_sound);
			  USART_Transmit('*'); 
			  elapsedTime =0;
  soundSum = 0;
  tempSum = 0;
			}
		adcT_result = adc_read(7);      // read adc value - temperature at PA3
		adcS_result = adc_read(1);
		adcDC_result = adc_read(3);

		temperature = adcT_result/3.8;   // finding the temperature (5/1023 = 4.88; 10/4.88 = 2.04)
		sound = abs(adcS_result-adcDC_result);     // Find Sound Value in db
		sound = 20*log(sound); // step size of ADC = 48.8 mv ; Sensitivity of Mic 0.005012 48.8mv/0.005012 = 9.756 dB - Gain of ckt
		
		tempSum += temperature;
soundSum += sound;
elapsedTime++;
  
		}
	}
}


 


