 
	#ifndef F_CPU
	#define F_CPU 1600000UL
	#endif
	#include <avr/io.h>
	#include <string.h>
	#include <util/delay.h> // needed for lcd_lib
	

	//**********************************************************
 
	#define LCD_DATA 	PORTC //LCD data port
 	#define ctrl 		PORTB
	#define en 			PB2 //enable signal
	#define rw 			PB1 //read/write signal
	#define rs 			PB0 //resister select signal
	

	//**********************************************************

	void LCD_cmd(unsigned char cmd);
	void init_LCD(void);
	void LCD_write(unsigned char data);
	void LCD_write_string(unsigned char *str);
	void LCDGotoXY(uint8_t x, uint8_t y);
	void lcd_clear(void);
	//**********************************************************

	#define LCD_LINES			2	//visible lines
	#define LCD_LINE_LENGTH		16	//line length (in characters)
	#define LCD_DDRAM           7	//DB7: set DD RAM address
	// cursor position to DDRAM mapping
	#define LCD_LINE0_DDRAMADDR		0x00
	#define LCD_LINE1_DDRAMADDR		0x40
	#define LCD_LINE2_DDRAMADDR		0x14
	#define LCD_LINE3_DDRAMADDR		0x54

	//**********************************************************

	void init_LCD(void)
	{
 
	 LCD_cmd(0x38); //initializtion of 16X2 LCD in 8bit mode
	 _delay_ms(1);
 
	 LCD_cmd(0x01); //clear LCD
	 _delay_ms(1);
 
	 LCD_cmd(0x0E); //cursor ON
	 _delay_ms(1);
 
	 LCD_cmd(0x80); // ---8 go to first line and --0 is for 0th position
	 _delay_ms(1);
	 return;
	}
 	
	void lcd_clear(void)
	{
	    LCD_cmd(0x01);
	    _delay_ms(5);
	}
 
	 void LCD_cmd(unsigned char cmd)
	 {
	 LCD_DATA=cmd;
	 ctrl =(0<<rs)|(0<<rw)|(1<<en); // making RS and RW as LOW and EN as HIGH
	 _delay_ms(1);
	 ctrl =(0<<rs)|(0<<rw)|(0<<en); // making RS, RW , LOW and EN as LOW
	 _delay_ms(50);
	 return;
	 }
 
 
	 void LCD_write(unsigned char data)
	 {

	 LCD_DATA= data;
	 ctrl = (1<<rs)|(0<<rw)|(1<<en); // making RW as LOW and RS, EN as HIGH
	 _delay_ms(1);
	 ctrl = (1<<rs)|(0<<rw)|(0<<en); // making EN and RW as LOW and RS HIGH
	 _delay_ms(5); // give a 10 milli second delay to get thigs executed
	 return ;
	 }

 
	 void LCD_write_string(unsigned char *str) //take address vaue of the string in pionter *str
	 {
	 int i=0;
	 while(str[i]!='\0') // loop will go on till the NULL charaters is soon in string 
	 {
	 LCD_write(str[i]); // sending data on CD byte by byte
	 i++;
	 }
	 return;
	 }

	 void LCDGotoXY(uint8_t x, uint8_t y)	//Cursor to X Y position
	{
		register uint8_t DDRAMAddr;
		// remap lines into proper order
		switch(y)
		{
		case 0: DDRAMAddr = LCD_LINE0_DDRAMADDR+x; break;
		case 1: DDRAMAddr = LCD_LINE1_DDRAMADDR+x; break;
		case 2: DDRAMAddr = LCD_LINE2_DDRAMADDR+x; break;
		case 3: DDRAMAddr = LCD_LINE3_DDRAMADDR+x; break;
		default: DDRAMAddr = LCD_LINE0_DDRAMADDR+x;
		}
		// set data address
		LCD_cmd(1<<LCD_DDRAM | DDRAMAddr);
	
	}
